package main

import (
	"github.com/gin-contrib/sessions"
	"github.com/gin-gonic/gin"
	"fmt"
)

type AddUserReq struct {
	Account  string
	Name     string
	Password string
	Depart   string
	Role     string
}

type AddUserResp struct {
	Code    string `json:"code"`
	State   string `json:"state"`
	Message string `json:"message"`
}

func addUser(ctx *gin.Context) {
	var req AddUserReq
	var resp AddUserResp
	resp.State = "失败"
	defer func() {
		ctx.JSON(200, resp)
	}()

	session := sessions.Default(ctx)
	if !checkLogin(session) {
		resp.Message = "未登录或会话超时"
		return
	}
	req.Account = ctx.PostForm("account")
	req.Name = ctx.PostForm("name")
	req.Password = ctx.PostForm("password")
	req.Depart = ctx.PostForm("depart")
	req.Role = ctx.PostForm("role")

	if req.Account == "" || len(req.Account) < 6 {
		resp.Message = "用户名不能低于6字节"
		return
	}

	if req.Name == "" || len(req.Name) < 2 {
		resp.Message = "用户姓名不能少于2字节"
		return
	}

	if req.Password == "" || len(req.Password) < 8 {
		resp.Message = "密码不能低于8字节"
		return
	}

	optAccount := session.Get("account").(string)

	rows, err := fileMgr.Mysql.Query("call p_sys_add_user(?,?, ?,?,?,?)",
		optAccount, req.Account, req.Name, req.Depart, req.Password, req.Role)
	if err != nil || rows == nil {
		resp.Message = fmt.Sprintf("数据库失败: %s", err);
		return
	}
	defer rows.Close();
	if !rows.Next() {
		resp.Message = "获取结果集失败"
		return
	}
	var ret int
	rows.Scan(&ret)
	if ret != 0 {
		resp.Message = "添加用户，请确保您是超级管理员"
		return
	}
	resp.State = "成功"
	resp.Message = "添加成功"
	resp.Code = "成功"
}
