class Pagebar extends HTMLElement {
    // pg-cur: current page number
    // pg-count: page count
    // pg-btn-count: page buttons count
    // pg-callback: click callback
    // pg-size: get page size
    static get observedAttributes() {
        return ['pg-cur', 'pg-count', 'pg-btn-count', 'pg-callback', 'pg-size'];
    }
    constructor() {
        super();
        let shadow = this.attachShadow({mode: "open"});
        let style = document.createElement("link");
        let domNode = document.createElement("div");
        shadow.appendChild(style);
        shadow.appendChild(domNode);
        domNode.className = "pagebar";
        style.setAttribute("rel", "stylesheet");
        style.setAttribute("href", "css/page-bar.css");
        this.lastTime = 0;
    }

    connectedCallback() {
        // this.shadowRoot.querySelector(".pagebar").innerHTML = "";
    }

    disconnectedCallback() {}

    adoptedCallback() {}

    attributeChangedCallback(name, oldvalue, newvalue) {
        if (name == "pg-cur" || name == "pg-count" || name == "pg-btn-count") {
            this.updateBar();
        }
    }

    updateBar() {
        let pgid = this.getAttribute("pg-cur");
        let pgcount = this.getAttribute("pg-count");
        let btns = this.getAttribute('pg-btn-count');
        if (pgid == "" || pgcount == "" || btns == "") {
            return;
        }
        let d = new Date();
        if (d.getTime() - this.lastTime < 500) {
            return;
        }
        this.lastTime = d.getTime();
        pgid = parseInt(pgid);
        pgcount = parseInt(pgcount);
        btns = parseInt(btns);
        if (pgid < 1) {
            pgid = 0;
        }
        if (pgcount < 0) {
            pgcount = 0;
        }
        if (btns < 1) {
            btns = 10;
        }

        let pgcenter = parseInt(btns / 2);
        let pgstart = pgid - pgcenter;
        let pgend   = pgid + pgcenter;
        if (pgstart < 1) {
            pgstart = 1;
            pgend = btns;
        }
        if (pgend > pgcount) {
            pgend = pgcount;
            pgstart = pgend - btns + 1;
        }
        if (pgstart < 1) {
            pgstart = 1;
        }

        let pbar = this.shadowRoot.querySelector(".pagebar");
        pbar.innerHTML = "";
        pbar.appendChild(this.createPageSize());
        pbar.appendChild(this.createButton(1, "第1页"));
        for (let i = pgstart; i <= pgend; ++i) {
            let active = false;
            if (pgid == i) {
                active = true;
            }
            pbar.appendChild(this.createButton(i, "" + i, active));
        }
        pbar.appendChild(this.createButton(pgcount, "共" + pgcount + "页"));
    }

    createPageSize() {
        let sizes = [15, 30, 60, 120, 240];
        let sel = document.createElement("select");
        let ps = this.getAttribute("pg-size");
        if (ps == "" || ps == undefined || ps == null) {
            ps = "" + sizes[0];
        }
        sel.className = "page-size";
        sel.setAttribute("name", "page-size");
        let me = this;
        for (let i = 0; i < sizes.length; ++i) {
            let opt = document.createElement("option");
            opt.value = "" + sizes[i];
            opt.textContent = "每页 " + sizes[i] + " 条";
            sel.appendChild(opt);
            opt.addEventListener("click", function(ev) {
                me.setAttribute("pg-size", this.value);
            });
            if (opt.value == ps) {
                opt.selected = true;
            }
        }
        return sel;
    }

    getPageSize() {
        let sel = this.shadowRoot.querySelector(".page-size");
        return sel.value;
    }

    createButton(page, title, active = false) {
        let me = this;
        let domNode = document.createElement("div");
        if (active) {
            domNode.className = "page-button active";
        } else {
            domNode.className = "page-button";
        }
        domNode.dataIndex = "" + page;
        domNode.textContent = title;
        domNode.addEventListener("click", function(ev) {
            ev.data = this;
            me.gotoPage(ev);
        }, false);
        return domNode;
    }

    gotoPage(event) {
        let pageid = event.data.dataIndex;
        let cbstr = this.getAttribute("pg-callback");
        if (!cbstr || cbstr == "") {
            event.preventDefault();
            return;
        }
        let pgsize = this.getPageSize();
        let cb = eval(cbstr);
        if (typeof cb == "function") {
            cb({"bar": this, "event": event, "pageid": pageid, "pagesize": pgsize});
        }
    }
}

customElements.define('f-page-bar', Pagebar);