package main

import (
	"crypto/md5"
	"fmt"
	"time"
	"unsafe"

	"github.com/gin-contrib/sessions"
	"github.com/gin-gonic/gin"
)

type LoginMenu struct {
	Name  string `json:"name"`  // 菜单名称
	Title string `json:"title"` // 菜单标题
	Url   string `json:"url"`   // 菜单url
	Blank string `json:"blank"` // 是否新窗口打开
}

type LoginResp struct {
	State   string      `json:"state"`   // 成功返回 "成功"
	Code    string      `json:"code"`    // 同 State
	Message string      `json:"message"` // 错误消息
	Account string 		`json:"account"` // 账号
	Name    string      `json:"name"`    // 用户姓名
	Role    string      `json:"role"`    // 用户角色
	Icon    string      `json:"icon"`    // 用户图像
	Menus   []LoginMenu `json:"menus"`   // 用户菜单
	Roles   []string 	`json:"roles"`	 // 所有role
}

// func (resp *LoginResp) String() string {
// 	b, _ := json.Marshal(resp);
// 	return *(*string)(unsafe.Pointer(&b));
// }

func Md5(plain string) string {
	data := *((*[]byte)(unsafe.Pointer(&plain)))
	return fmt.Sprintf("%x", md5.Sum(data))
}

func expired(session sessions.Session) bool {
	loginTimeX := session.Get("time")
	if loginTimeX == nil {
		return true;
	}
	loginTime := loginTimeX.(int64);
	nowTime := time.Now().Unix()
	if nowTime-loginTime > 24*60*60 {
		return true
	}
	return false
}

func checkLogin(session sessions.Session) bool {
	if expired(session) {
		return false
	}
	
	if session.Get("account").(string) == "" {
		return false
	}
	return true
}

func login(ctx *gin.Context) {
	var resp LoginResp
	defer func(resp *LoginResp) {
		ctx.JSON(200, resp)
	}(&resp)
	resp.Menus = make([]LoginMenu, 0, 16)
	resp.Roles = make([]string, 0, 3);
	resp.State = "失败"
	resp.Code = "失败"

	user := ctx.PostForm("username")
	pass := ctx.PostForm("password")
	resp.Account = user

	// 防机器人
	if ctx.PostForm("requiredName") != "" || ctx.PostForm("requiredPwd") != "" {
		resp.Message = "登录失败";
		return;
	}

	if user == "" || pass == "" {
		resp.Message = "用户名或密码不能为空"
		return
	}
	encpass := Md5(pass)
	rows, err := fileMgr.Mysql.Query("call p_sys_login(?,?)", user, encpass)
	if err != nil || rows == nil {
		resp.Message = fmt.Sprintf("%s", err)
		return
	}
	defer rows.Close()
	if !rows.Next() {
		resp.Message = fmt.Sprintf("%s", err)
		return
	}
	var ret int
	rows.Scan(&ret, &resp.Name, &resp.Role, &resp.Icon)
	if ret != 0 {
		resp.Message = "登录失败"
		return
	}
	if !rows.NextResultSet() {
		resp.Message = "获取用户菜单失败"
		return
	}
	for rows.Next() {
		var menu LoginMenu
		rows.Scan(&menu.Name, &menu.Title, &menu.Url)
		resp.Menus = append(resp.Menus, menu)
	}
	if !rows.NextResultSet() {
		resp.Message = "获取角色们失败"
		return;
	}
	for rows.Next() {
		var r string;
		rows.Scan(&r);
		resp.Roles = append(resp.Roles, r);
	}
	resp.State = "成功"
	resp.Code = "成功"
	resp.Message = "成功"

	// write session
	session := sessions.Default(ctx)
	session.Set("account", user)
	session.Set("name", resp.Name)
	session.Set("role", resp.Role)
	session.Set("time", time.Now().Unix())
	session.Save()
}
