package main

import (
	"database/sql"
	"fmt"
	"io"
	"os"
	"path/filepath"
	"time"

	"github.com/gin-contrib/sessions"
	"github.com/gin-contrib/sessions/cookie"
	"github.com/gin-gonic/gin"
	_ "github.com/yireyun/mysql"
)

const (
	FileRoot       = "2057de239adcd8814f5e08f5b40f04f8"
	CookieStoreKey = "6abe6555342f38867517392f2e4bc241"
	DownloadPrefix = "/5dea50b5cb787d64aaecb43e896a18eb"
)

type FileMgr struct {
	Cfg    *Config
	Router *gin.Engine
	Mysql  *sql.DB
	Store  cookie.Store
}

func NewFileMgr() *FileMgr {
	var err error
	var mgr *FileMgr = new(FileMgr)
	mgr.Cfg = NewConfig()
	mgr.Cfg.LoadConfig()

	if mgr.Cfg.LogFile != "" {
		logfile, err := os.Create(mgr.Cfg.LogFile)
		if err != nil {
			fmt.Printf("can not open file: %s\n", mgr.Cfg.LogFile)
			return nil
		}
		gin.DefaultWriter = io.MultiWriter(logfile)
	}

	mgr.Router = gin.New()
	mgr.Router.Use(gin.LoggerWithFormatter(func(param gin.LogFormatterParams) string {
		return fmt.Sprintf("%s - [%s] \"%s %s %s %d %s \"%s\" %s\"\n",
			param.ClientIP,
			param.TimeStamp.Format(time.RFC1123),
			param.Method,
			param.Path,
			param.Request.Proto,
			param.StatusCode,
			param.Latency,
			param.Request.UserAgent(),
			param.ErrorMessage,
		)
	}))
	mgr.Store = cookie.NewStore([]byte(CookieStoreKey))
	mgr.Router.Use(sessions.Sessions("mysession", mgr.Store))

	mgr.Router.Use(gin.Recovery())

	err, mgr.Mysql = initDb(&mgr.Cfg.Db)
	if err != nil {
		fmt.Printf("open db failed: %s\n", err)
		return nil
	}
	return mgr
}

func (fm *FileMgr) loadStaticFiles() {
	filepath.Walk("./html", func(path string, f os.FileInfo, err error) error {
		if f == nil {
			return nil
		}
		if f.IsDir() {
			return nil
		}
		if path[4:] == "/index.html" {
			fm.Router.StaticFile("/", path);
		}
		fm.Router.StaticFile(path[4:], path)
        return nil
	})
}

func (fm *FileMgr) loadInterface() {
	fm.Router.POST("/api/login", login)
	fm.Router.POST("/api/logout", logout)
	fm.Router.GET("/api/logout", logout);
	fm.Router.POST("/api/add-user", addUser)
	fm.Router.POST("/api/mod-user-info", modifyUserInfo)
	fm.Router.POST("/api/get-user-info/:howmany", getUserInfo)

	fm.Router.POST("/api/upload", uploadFile)
	fm.Router.OPTIONS("/api/upload", testUpload);
	fm.Router.GET(fmt.Sprintf("%s/:fpath/:fname", DownloadPrefix), downloadFile)
	fm.Router.POST("/api/get-file-info", getFileInfo)
	fm.Router.POST("/api/mod-file-info", modifyFileInfo)
	fm.Router.GET("/search", searchFile)
	fm.Router.POST("/search", searchFile)
	fm.Router.POST("/api/delfile", removeFile);
}

func (fm *FileMgr) loadRouts() {
	fm.loadStaticFiles()
	fm.loadInterface()
}

func (fm *FileMgr) Start() {
	fm.loadRouts()
	fm.Router.Run(fm.Cfg.Address())
}

var fileMgr *FileMgr = nil

func main() {
	fileMgr = NewFileMgr()
	if fileMgr == nil {
		fmt.Printf("start failed.\n")
		return
	}

	fileMgr.Start()
}
