package main

import (
	"fmt"
	"strconv"

	"github.com/gin-contrib/sessions"
	"github.com/gin-gonic/gin"
)

type QueryReq struct {
	Query    string `json:"query"`
	PageSize int    `json:"pagesize"`
	Pageid   int    `json:"pageid"`
}

type QueryRespItem struct {
	Id       string `json:"id"`
	Name     string `json:"name"`
	Fname    string `json:"fname"`
	Product  string `json:"product"`
	Type     string `json:"type"`
	Time     string `json:"time"`
	Uploader string `json:"uploader"`
	Download string `json:"download"`
	Editable bool   `json:"editable"`
}

type QueryResp struct {
	State   string `json:"state"`
	Code    string `json:"code"`
	Message string `json:"message"`

	PageId    int `json:"pageid"`
	TotalRows int `json:"totalRows"`
	TotalPage int `json:"totalPage"`
	PageSize  int `json:"pagesize"`

	Rows []QueryRespItem `json:"rows"`
}

func searchFile(ctx *gin.Context) {
	var req QueryReq
	var resp QueryResp
	resp.State = "失败"
	resp.Code = "失败"
	resp.Rows = make([]QueryRespItem, 0, 1)
	defer func() {
		ctx.JSON(200, resp)
	}()
	session := sessions.Default(ctx)
	if !checkLogin(session) {
		resp.Message = "请重新登录"
		return
	}
	// req.query = ctx.PostForm("query")
	// req.pageSize = ctx.PostForm("pagesize")
	// req.pageid = ctx.PostForm("pageid")
	perr := ctx.BindJSON(&req)
	if perr != nil {
		resp.Message = fmt.Sprintf("parse param failed: %s", perr)
		return
	}

	if req.Query == "" {
		req.Query = ctx.Query("query")
	}

	if req.PageSize == 0 {
		ps := ctx.DefaultQuery("pagesize", "15")
		req.PageSize, _ = strconv.Atoi(ps)
	}

	if req.Pageid == 0 {
		pn := ctx.DefaultQuery("pageid", "1")
		req.Pageid, _ = strconv.Atoi(pn)
	}

	account := session.Get("account").(string)
	rows, err := fileMgr.Mysql.Query("call p_web_page(?, ?, ?, ?)", account, req.Query,
		req.Pageid, req.PageSize)
	if err != nil || rows == nil {
		resp.Message = "数据库操作失败"
		return
	}
	defer rows.Close();
	if !rows.Next() {
		resp.Message = "获取总计信息失败"
		return
	}
	rows.Scan(&resp.PageId, &resp.PageSize, &resp.TotalPage, &resp.TotalRows)
	resp.Rows = make([]QueryRespItem, 0, resp.PageSize)

	if !rows.NextResultSet() {
		resp.Message = "获取结果失败"
		return
	}

	for rows.Next() {
		var row QueryRespItem
		rows.Scan(&row.Id, &row.Name, &row.Fname, &row.Product, &row.Type, &row.Time, &row.Uploader,
			&row.Editable, &row.Download)
		resp.Rows = append(resp.Rows, row)
	}
	resp.State = "成功"
	resp.Code = "成功"
	resp.Message = "成功"
}
