package main

import (
	"fmt"
	"time"

	"github.com/gin-contrib/sessions"
	"github.com/gin-gonic/gin"

	"os"
)

func pathExists(path string) bool {
	_, err := os.Stat(path)
	if err == nil {
		return true
	}
	if os.IsNotExist(err) {
		return false
	}
	return false
}

func getFileRealName(name string) (string, string, string) {
	now := time.Now()
	strYear := Md5(fmt.Sprintf("%04d", now.Year()))
	strName := Md5(name)
	fdir := fmt.Sprintf("./%s/%s", FileRoot, strYear)
	if !pathExists(fdir) {
		os.Mkdir(fdir, os.ModePerm)
	}
	return fmt.Sprintf("%s/%s", fdir, strName), strYear, strName
}

type UploadReq struct {
	FileName string
	Title    string
	Type     string
	Product  string

	ProductType string
	Version     string
	SvnPath     string
	Tester      string
	Text        string

	RealPath string
	Link     string
}

type UploadResp struct {
	State   string `json:"state"`
	Code    string `json:"code"`
	Message string `json:"message"`
}

func testUpload(ctx *gin.Context) {
	ctx.String(200, "success");
}

func uploadFile(ctx *gin.Context) {
	var req UploadReq
	var resp UploadResp = UploadResp{"失败", "失败", "失败"}
	defer func() {
		ctx.JSON(200, resp)
	}()
	session := sessions.Default(ctx)
	if !checkLogin(session) {
		resp.Message = "请重新登录"
		return
	}

	req.Title = ctx.PostForm("title")
	req.Type = ctx.PostForm("type")
	req.Product = ctx.PostForm("product")
	req.ProductType = ctx.PostForm("product-type")
	req.Version = ctx.PostForm("version")
	req.SvnPath = ctx.PostForm("svnpath")
	req.Tester = ctx.PostForm("tester")
	req.Text = ctx.PostForm("text")

	file, err := ctx.FormFile("file")
	if err != nil {
		resp.Message = fmt.Sprintf("%s", err)
		return
	}
	var hashPath, hashName string
	req.FileName = file.Filename
	req.RealPath, hashPath, hashName = getFileRealName(file.Filename)
	req.Link = fmt.Sprintf("%s/%s/%s", DownloadPrefix, hashPath, hashName)

	account := session.Get("account").(string)
	// 数据库鉴权
	rows, err := fileMgr.Mysql.Query("call p_web_upload(?, ?,?,?,?,?,?, ?,?,?,?,?)", account, req.FileName, req.RealPath,
		req.Link, req.Title, req.Type, req.Product, req.ProductType, req.Version, req.SvnPath, req.Tester, req.Text)
	if err != nil || rows == nil {
		resp.Message = "数据库操作失败"
		return
	}
	defer rows.Close();

	if !rows.Next() {
		resp.Message = "获取数据库结果失败"
		return
	}
	var ret int
	var fileId int64
	rows.Scan(&ret, &fileId)
	if ret != 0 {
		resp.Message = "权限不足，上传失败"
		return
	}
	ctx.SaveUploadedFile(file, req.RealPath)
	resp.State = "成功"
	resp.Code = "成功"
	resp.Message = "上传成功"
}
